package es.us.eii.pfc.jcrbib.data;

import java.util.SortedSet;

import es.us.eii.pfc.jcrbib.domain.MyArticle;
import es.us.eii.pfc.jcrbib.domain.ex.ItemAlreadyExistsException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemNotExistsException;

/**
 * Interfaz para el almacenamiento de artculos de usuario
 * 
 * 
 * 
 */
public interface IMyArticleDAO {

	/**
	 * Inserta un artculo en el sistema
	 * 
	 * @param myArticle
	 *            Articulo a insertar
	 * @param userName
	 *            Nombre del usuario al cual queremos aadir el artculo
	 */
	public void addMyArticle(MyArticle myArticle, String userName)
			throws ItemAlreadyExistsException;

	/**
	 * Mtodo para actualizar un artculo de usuario. El artculo debe existir
	 * para que funcione corretamente
	 * 
	 * @param myArticle
	 *            Artculo a modificar
	 * @param userName
	 *            Usuario que posee el artculo
	 * @return El artculo modificado
	 * @throws ItemNotExistsException
	 *             Si el artculo no existe se eleva esta excepcin, porque no
	 *             se puede actualizar
	 */
	public MyArticle updateMyArticle(MyArticle myArticle, String userName)
			throws ItemNotExistsException;

	/**
	 * Elimina el artculo con el ID especificado de la biblioteca del usuario
	 * 
	 * @param id
	 *            ID del artculo a eliminar
	 * @param userName
	 *            Usuario que posee el artculo
	 * @return El artculo eliminado
	 * @throws ItemNotExistsException
	 *             Se eleva esta excepcin si se pretende eliminar un artculo
	 *             que no existe en la biblioteca del usuario.
	 */
	public MyArticle removeMyArticle(String id, String userName)
			throws ItemNotExistsException;

	/**
	 * Mtodo para obtener un artculo de usuario
	 * 
	 * @param id
	 *            ID del artculo a obtener
	 * @param userName
	 *            Nombre del usuario poseedor del artculo
	 * @return El artculo deseado
	 * @throws ItemNotExistsException
	 *             Si se intenta obtener un artculo que no existe en la
	 *             biblioteca del usuario
	 */
	public MyArticle getMyArticle(String id, String userName)
			throws ItemNotExistsException;

	/**
	 * 
	 * Mtodo que retorna todos los artculos de un usuario
	 * 
	 * @param userName
	 *            Usuario del cual queremos obtener sus artculos
	 * @return Un Conjunto Ordenado (SortedSet) de artculos
	 */
	public SortedSet<MyArticle> getMyArticles(String userName);

	/**
	 * Establece en bloque los artculos de un usuario. Los artculos
	 * anteriores, si es que los tuviere, son eliminados por los enviados con
	 * este mtodo.
	 * 
	 * @param myArticles
	 *            Conjunto Ordenado (SortedSet) con los artculos que compondrn
	 *            la biblioteca del usuario
	 * @param userName
	 *            Usuario al cual modificaremos su biblioteca
	 */
	public void setMyArticles(SortedSet<MyArticle> myArticles, String userName);

	/**
	 * Mtodo para la obtencin rpida de todas las categorias de un artculo.
	 * No es lo mismo que obtener las categoras de un artculo concreto, porque
	 * en funcin del usuario estas categoras pueden variar
	 * 
	 * @param id
	 *            ID del artculo cuyas categoras queremos conocer
	 * @return Un conjunto ordenado (SortedSet) de Tag, con su informacin
	 *         completa.
	 */
	public SortedSet<String> getAllMyArticleTags(String id);
}