package es.us.eii.pfc.jcrbib.data;

import java.util.SortedSet;

import es.us.eii.pfc.jcrbib.domain.User;
import es.us.eii.pfc.jcrbib.domain.ex.ItemAlreadyExistsException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemNotExistsException;

/**
 * Interfaz para el almacenamiento y consulta de usuarios del sistema
 * 
 * 
 * 
 */
public interface IUserDAO {

	/**
	 * Mtodo que indica si existe el usuario pasado como parmetro
	 * 
	 * @param userName
	 *            Nombre del usuario a consultar
	 * @return Devuelve true si el usuario existe
	 */
	public boolean existsUser(String userName);

	/**
	 * Mtodo para aadir un usuario al sistema
	 * 
	 * @param user
	 *            Especificacin del usuario a insertar
	 * @throws ItemAlreadyExistsException
	 *             Se eleva esta excepcin si ya existe un usuario con este
	 *             nombre en el sistema
	 */
	public void addUser(User user) throws ItemAlreadyExistsException;

	
	/**
	 * Mtodo para actualizar un usuario del sistema
	 * 
	 * @param user Nueva informacin del usuario a actualizar
	 * @return Devuelve el usuario actualizado
	 * @throws ItemNotExistsException Se eleva esta excepcin si no existe el usuario
	 */
	public User updateUser(User user) throws ItemNotExistsException;

	/**
	 * Mtodo para la obtencin de un usuario
	 * 
	 * @param name Nombre del usuario a obtener
	 * @return El objeto de tipo User con la informacin del usuario
	 * @throws ItemNotExistsException Se eleva esta excepcin si no existe el usuario
	 */
	public User getUser(String name) throws ItemNotExistsException;

	/**
	 * Mtodo para eliminar un usuario del sistema
	 * 
	 * @param name Nombre del usuario a eliminar
	 * @return El usuario eliminado
	 * @throws ItemNotExistsException Si el usuario no existe en el sistema
	 */
	public User removeUser(String name) throws ItemNotExistsException;

	/**
	 * Mtodo para la obtencin de todos los usuarios del sistema
	 * 
	 * @return Un conjunto ordenado (SortedSet) con los usuarios del sistema
	 */
	public SortedSet<User> getUsers();
}
