package es.us.eii.pfc.jcrbib.data;

import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.LOGGERNAME;

import java.util.HashMap;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.apache.log4j.Logger;

import es.us.eii.pfc.jcrbib.domain.MyArticle;
import es.us.eii.pfc.jcrbib.domain.Tag;
import es.us.eii.pfc.jcrbib.domain.User;
import es.us.eii.pfc.jcrbib.domain.ex.InvalidParameterException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemAlreadyExistsException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemNotExistsException;
import es.us.eii.pfc.jcrbib.utils.NodeUtils;
import es.us.eii.pfc.jcrbib.utils.RepositoryUtils;
import es.us.eii.pfc.jcrbib.utils.SessionUtils;

/**
 * 
 * Implementacin para el almacenamiento con JCR de los artculos de usuario
 * 
 * 
 * 
 */
public class JcrMyArticleDAO implements IMyArticleDAO {

	// Externalizacin de cadenas a la clase MessagesDAO y su properties

	private static String PRIORITY_PROPERTY_NAME = MessagesDAO
			.getString("JcrMyArticleDAO.priority_name_property");

	private static String URL_PROPERTY_NAME = MessagesDAO
			.getString("JcrMyArticleDAO.url_name_property");

	private static String TITLE_PROPERTY_NAME = MessagesDAO
			.getString("JcrMyArticleDAO.title_name_property");

	protected static String TAG_NAME = MessagesDAO
			.getString("JcrTagDAO.tags_name_tag"); //$NON-NLS-1$

	protected static String AUTHORS_NAME = MessagesDAO
			.getString("JcrMyArticleDAO.authors_name_tag"); //$NON-NLS-1$

	// Log para registrar las acciones
	private static Logger log = Logger.getLogger(LOGGERNAME);

	protected JcrMyArticleDAO() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IMyArticleDAO#addMyArticle(es.us.eii.pfc.jcrbib.domain.MyArticle,
	 *      java.lang.String)
	 */
	public void addMyArticle(MyArticle myArticle, String userName)
			throws ItemAlreadyExistsException {
		Session s = SessionUtils.getSession();
		addMyArticle(myArticle, userName, s);
		s.logout();
		log.debug("Cerrando session");
	}

	/*
	 * (non-Javadoc) Mtodo que implementa la accin de aadir artculo
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrMyArticleDAO#addMyArticle(es.us.eii.pfc.jcrbib.domain.MyArticle,
	 *      java.lang.String)
	 */
	protected void addMyArticle(MyArticle myArticle, String userName, Session s)
			throws ItemAlreadyExistsException {
		Node init;
		try {
			// Registramos que estamos insertando un artculo en el log
			log.debug(MessagesDAO
					.getString("JcrMyArticleDAO.messages_inserting_article")); //$NON-NLS-1$

			// Obtenemos el nodo inicial
			init = RepositoryUtils.getInitNode(s);

			String myArticlesPath = JcrUserDAO.USERS_NODE_NAME
					+ "/" + NodeUtils.encodeUserName(userName) + //$NON-NLS-1$
					"/" + JcrUserDAO.MY_ARTICLES_NODE_NAME; //$NON-NLS-1$

			// Obtenemos el nodo de los artculos de usuario
			Node myArticlesNode = init.getNode(myArticlesPath);

			if (myArticlesNode.hasNode(myArticle.getId())) {
				throw new ItemAlreadyExistsException("MyArticle "
						+ myArticle.getId() + " already " + "exists for user "
						+ userName);
			}

			// Aadimos el nuevo nodo
			Node newMyArticleNode = myArticlesNode.addNode(myArticle.getId());

			// Establecemos las propiedades
			newMyArticleNode.setProperty(TITLE_PROPERTY_NAME, myArticle
					.getTitle()); //$NON-NLS-1$
			newMyArticleNode.setProperty(URL_PROPERTY_NAME, myArticle.getUrl());
			newMyArticleNode.setProperty(PRIORITY_PROPERTY_NAME,
					(long) myArticle.getPriority()); //$NON-NLS-1$

			// Insertamos los autores
			Node authorsNode = newMyArticleNode.addNode(AUTHORS_NAME);
			SortedSet<String> authors = myArticle.getAuthors();
			for (String author : authors) {
				authorsNode.addNode(NodeUtils.encodeAuthorName(author));
			}

			Node tagsNode = newMyArticleNode.addNode(TAG_NAME);

			JcrTagDAO tagDAO = new JcrTagDAO();
			Tag tag;
			for (String tagName : myArticle.getTags()) {

				// Para cada tag, vemos si existe. Si no, lo creamos
				if (!tagDAO.existsTag(tagName, s)) {
					tag = new Tag(tagName);
					try {
						tagDAO.addTag(tag, s);
					}
					// A este catch nunca debera llegar
					catch (ItemAlreadyExistsException e) {
						log.error(e);
					}
				} else {
					tag = tagDAO.getTag(tagName, s);
					tag.incWeight(); 
					try {
						tagDAO.updateTag(tag, s);
					} catch (ItemNotExistsException e) {
						// En este catch no debera entrar
						log.error(e);
					}
				}
				// y aadimos el tag correspondiente a nuestra lista
				String encTagName = NodeUtils.encodeTagName(tagName);
				if (!tagsNode.hasNode(encTagName)) {
					tagsNode.addNode(encTagName);
				}
			}

			// Almacenamos los cambios
			s.save();

			// Si hemos llegado aqu, se ha insertado el artculo con xito
			log
					.debug(MessagesDAO
							.getString("JcrMyArticleDAO.messages_succesfully_inserted_article")); //$NON-NLS-1$
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IMyArticleDAO#removeMyArticle(java.lang.String,
	 *      java.lang.String)
	 */
	public MyArticle removeMyArticle(String id, String userName)
			throws ItemNotExistsException {
		Session s = SessionUtils.getSession();
		MyArticle myArticle = null;
		// Realizamos la accin
		myArticle = removeMyArticle(id, userName, s);
		s.logout();
		log.debug("Cerrando session");
		return myArticle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * Mtodo que implementa la accin de eliminar artculo
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrMyArticleDAO#removeMyArticle(java.lang.String,
	 *      java.lang.String)
	 */
	protected MyArticle removeMyArticle(String id, String userName, Session s)
			throws ItemNotExistsException {
		Node init;
		MyArticle myArt = null;
		try {
			// Obtenemos el nodo inicial
			init = RepositoryUtils.getInitNode(s);
			String myArticlesPath = JcrUserDAO.USERS_NODE_NAME
					+ "/" + NodeUtils.encodeUserName(userName) + //$NON-NLS-1$
					"/" + JcrUserDAO.MY_ARTICLES_NODE_NAME; //$NON-NLS-1$

			// Obtenemos el nodo de artculos de usuario
			Node myArticlesNode = init.getNode(myArticlesPath);

			// Si existe el nodo (artculo) con ese id
			if (myArticlesNode.hasNode(id)) {

				// Obtenemos el nodo
				myArt = getMyArticle(id, userName, s);

				// Actualizamos la informacin de las categoras
				SortedSet<String> tags = myArt.getTags();
				JcrTagDAO tagDAO = new JcrTagDAO();
				Tag tag;
				for (String tagAct : tags) {
					tag = tagDAO.getTag(tagAct, s);
					// Si la etiqueta tiene un peso mayor que 1
					if (tag.getWeight() > 1) {
						// reducimos su peso
						tag.redWeight();
						// y actualizamos
						tagDAO.updateTag(tag, s);
					} else {
						// En otro caso, la eliminamos
						tagDAO.removeTag(tagAct, s);
					}
				}

				// Una vez actualizada la informacin sobre categoras,
				// eliminamos el nodo
				myArticlesNode.getNode(id).remove();
			} else {
				throw new ItemNotExistsException("MyArticle id:" + id
						+ " not exists");
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}

		// Devolvemos el artculo eliminado
		return myArt;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * Realiza la accin de eliminar todos los artculos de un usuario. Es
	 * invocado al eliminar un usuario del sistema
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrUserDAO#removeUser(java.lang.String,
	 *      java.lang.String)
	 */
	protected void removeMyArticles(String userName, Session s)
			throws ItemNotExistsException {
		JcrUserDAO userDAO = new JcrUserDAO();
		User user = userDAO.getUser(userName, s);

		// Obtenemos la informacin de los tags del usuario para
		// actualizarla en el sistema
		Map<String, Integer> tagMapAct = new HashMap<String, Integer>();

		SortedSet<MyArticle> myArticles = user.getMyArticles();
		for (MyArticle myAct : myArticles) {
			for (String tag : myAct.getTags()) {
				if (tagMapAct.keySet().contains(tag)) {
					Integer intObj = tagMapAct.remove(tag);
					tagMapAct.put(tag, intObj + 1);
				} else {
					tagMapAct.put(tag, 1);
				}
			}
		}

		// Actualizamos los tags
		JcrTagDAO tagDAO = new JcrTagDAO();
		int newWeight;
		for (String lTag : tagMapAct.keySet()) {
			newWeight = tagDAO.getTagWeight(lTag, s) - tagMapAct.get(lTag);

			// Si el nuevo peso es mayor que 0, actualizamos
			if (newWeight > 0) {
				try {
					tagDAO.setTagWeight(lTag, newWeight, s);
				}
				// En estos catch's no deberia entrar
				catch (InvalidParameterException e) {
					log.error(e);
				} catch (ItemNotExistsException e) {
					log.error(e);
				}
			} else {
				// En otro caso, eliminamos la etiqueta
				try {
					tagDAO.removeTag(lTag, s);
				}
				// En este catch no deberia entrar
				catch (ItemNotExistsException e) {
					log.error(e);
				}
			}
		}

		// Y por ltimo, eliminamos todos los artculos (a nivel de nodo)
		Node init;
		try {
			init = RepositoryUtils.getInitNode(s);

			String myArticlesPath = JcrUserDAO.USERS_NODE_NAME
					+ "/" + NodeUtils.encodeUserName(userName) + //$NON-NLS-1$
					"/" + JcrUserDAO.MY_ARTICLES_NODE_NAME; //$NON-NLS-1$
			Node myArticlesNode = init.getNode(myArticlesPath);
			NodeIterator nt = myArticlesNode.getNodes();

			while (nt.hasNext()) {
				nt.nextNode().remove();
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IMyArticleDAO#setMyArticles(java.util.SortedSet,
	 *      java.lang.String)
	 */
	public void setMyArticles(SortedSet<MyArticle> myArticles, String userName) {
		Session s = SessionUtils.getSession();
		setMyArticles(myArticles, userName, s);
		s.logout();
		log.debug("Cerrando session");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * Establece un nuevo conjunto de artculos para el usuario especificado
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrMyArticleDAO#setMyArticles(java.util.SortedSet,
	 *      java.lang.String)
	 */
	protected void setMyArticles(SortedSet<MyArticle> myArticles,
			String userName, Session s) {
		Node init;
		try {
			// Obtenemos el nodo inicial
			init = RepositoryUtils.getInitNode(s);

			// Codificamos el nombre de usuario
			String encUserName = NodeUtils.encodeUserName(userName);
			String myArticlesPath = JcrUserDAO.USERS_NODE_NAME + "/"
					+ encUserName + "/" + JcrUserDAO.MY_ARTICLES_NODE_NAME;

			// Borramos todos los nodos (artculos) antiguos
			Node userMyArticles = init.getNode(myArticlesPath);
			NodeIterator nt = userMyArticles.getNodes();
			while (nt.hasNext()) {
				nt.nextNode().remove();
			}

			// Y ahora agregamos todos los nuevos artculos
			for (MyArticle myAct : myArticles) {
				try {
					addMyArticle(myAct, userName, s);
				} catch (ItemAlreadyExistsException e) {
					// En este catch no debera entrar porque
					// hemos eliminado todos los artculos del usuario
					log.error(e);
				}
			}

			// Salvamos los cambios
			s.save();
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IMyArticleDAO#updateMyArticle(es.us.eii.pfc.jcrbib.domain.MyArticle,
	 *      java.lang.String)
	 */
	public MyArticle updateMyArticle(MyArticle myArticle, String userName)
			throws ItemNotExistsException {
		removeMyArticle(myArticle.getId(), userName);
		try {
			addMyArticle(myArticle, userName);
		} catch (ItemAlreadyExistsException e) {
			// Esta excepcin no debera saltar dado que acabamos de eliminar
			// el artculo en el paso anterior
			log.error(e);
		}
		return myArticle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * Obtiene todos los artculos de un usuario
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrMyArticleDAO#setMyArticles(java.util.SortedSet,
	 *      java.lang.String)
	 */
	protected SortedSet<MyArticle> getMyArticles(String userName, Session s) {
		SortedSet<MyArticle> listMyArticles = new TreeSet<MyArticle>();
		Node init;
		try {
			// Obtenemos el nodo inicial
			init = RepositoryUtils.getInitNode(s);

			// Codificamos el nombre del usuario
			String encUserName = NodeUtils.encodeUserName(userName);
			String myArticlesPath = JcrUserDAO.USERS_NODE_NAME + "/"
					+ encUserName + "/" + JcrUserDAO.MY_ARTICLES_NODE_NAME;

			// Obtenemos el nodo de artculos del usuario
			Node myArticlesNode = init.getNode(myArticlesPath);

			// Recorremos todos los artculos y los vamos almacenando
			NodeIterator nt = myArticlesNode.getNodes();
			while (nt.hasNext()) {
				try {
					// El nombre del nodo es el ID del artculo
					listMyArticles.add(getMyArticle(nt.nextNode().getName(),
							userName, s));
				} catch (ItemNotExistsException e) {
					log.error(e);
				}
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}

		// Devolvemos los artculos
		return listMyArticles;
	}

	/*
	 * (non-Javadoc)
	 *  
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IMyArticleDAO#getMyArticle(java.lang.String,
	 *      java.lang.String)
	 */
	public MyArticle getMyArticle(String id, String userName)
			throws ItemNotExistsException {
		MyArticle myArticle = null;
		Session s = SessionUtils.getSession();
		myArticle = getMyArticle(id, userName, s);
		s.logout();
		log.debug("Cerrando session");
		return myArticle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * Metodo que obtiene un artculo de usuario
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrMyArticleDAO#getMyArticle(java.lang.String,
	 *      java.lang.String)
	 */
	protected MyArticle getMyArticle(String id, String userName, Session s)
			throws ItemNotExistsException {
		Node init;
		MyArticle myArticle = null;
		try {
			init = RepositoryUtils.getInitNode(s);
			String encUserName = NodeUtils.encodeUserName(userName);
			String myArticlesPath = JcrUserDAO.USERS_NODE_NAME + "/"
					+ encUserName + "/" + JcrUserDAO.MY_ARTICLES_NODE_NAME;
			Node userMyArticles = init.getNode(myArticlesPath);
			// Verificamos que existe el nodo
			if (userMyArticles.hasNode(id)) {
				Node myArticleNode = userMyArticles.getNode(id);

				// Obtenemos las propiedades
				String title = myArticleNode.getProperty(TITLE_PROPERTY_NAME)
						.getString();

				String url = myArticleNode.getProperty(URL_PROPERTY_NAME)
						.getString();

				int priority = (int) myArticleNode.getProperty(
						PRIORITY_PROPERTY_NAME).getLong();

				// Obtenemos los tags
				SortedSet<String> tags = new TreeSet<String>();
				NodeIterator nt = myArticleNode.getNode(TAG_NAME).getNodes();
				while (nt.hasNext()) {
					tags.add(NodeUtils.decodeTagName(nt.nextNode().getName()));
				}

				// y obtenemos los autores
				SortedSet<String> authors = new TreeSet<String>();
				NodeIterator nt2 = myArticleNode.getNode(AUTHORS_NAME)
						.getNodes();
				while (nt2.hasNext()) {
					authors.add(NodeUtils.decodeAuthorName(nt2.nextNode()
							.getName()));
				}

				// Cuando tenemos todos los valores, creamos el objeto
				try {
					myArticle = new MyArticle(id, title, url, authors, tags,
							priority);
				} catch (InvalidParameterException e) {
					log.error(e);
				}
			} else {
				throw new ItemNotExistsException("MyArticle id: " + id
						+ " not exists");
			}
		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
		return myArticle;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IMyArticleDAO#getMyArticles(java.lang.String)
	 */
	public SortedSet<MyArticle> getMyArticles(String userName) {
		SortedSet<MyArticle> myArticles = null;
		Session s = SessionUtils.getSession();
		myArticles = getMyArticles(userName, s);
		s.logout();
		log.debug("Cerrando session");
		return myArticles;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.IMyArticleDAO#getAllMyArticleTags(java.lang.String)
	 */
	public SortedSet<String> getAllMyArticleTags(String id) {
		SortedSet<String> allTags = null;
		Session s = SessionUtils.getSession();
		allTags = getAllMyArticleTags(id, s);
		s.logout();
		log.debug("Cerrando session");
		return allTags;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * Mtodo que obtiene todas las categoras de un artculo, recorriendo todos los usuarios
	 * 
	 * @see es.us.eii.pfc.jcrbib.data.JcrMyArticleDAO#getAllMyArticleTags(java.lang.String)
	 */
	protected SortedSet<String> getAllMyArticleTags(String id, Session s) {
		Node init;
		SortedSet<String> tagNames = new TreeSet<String>();
		try {
			// Obtenemos el nodo inicial
			init = RepositoryUtils.getInitNode(s);
			
			// Las rutas para usuarios y articulos
			String usersPath = JcrUserDAO.USERS_NODE_NAME + "/";
			String articlePath = JcrUserDAO.MY_ARTICLES_NODE_NAME + "/" + id;

			
			Node usersNode = init.getNode(usersPath);
			
			// Iteramos sobre los usuarios buscando el artculo
			NodeIterator nIt = usersNode.getNodes();
			Node userActNode;
			String userName;
			while (nIt.hasNext()) {
				userActNode = nIt.nextNode();
				userName = NodeUtils.decodeAuthorName(userActNode.getName());
				// Si el usuario tiene el artculo
				if (userActNode.hasNode(articlePath)) {
					try {
						// Aadimos todas las categoras de ese usuario
						tagNames
								.addAll(getMyArticle(id, userName, s).getTags());
					} catch (ItemNotExistsException e) {
						// Nunca debera llegar, la condicion previa as lo
						// indica
						log.error(e);
					}
				}
			}

		} catch (PathNotFoundException e) {
			log.error(e);
		} catch (RepositoryException e) {
			log.error(e);
		}
		return tagNames;
	}

}
