package es.us.eii.pfc.jcrbib.domain;

import java.util.SortedSet;
import java.util.UUID;

import es.us.eii.pfc.jcrbib.domain.ex.InvalidParameterException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemAlreadyExistsException;
import es.us.eii.pfc.jcrbib.domain.ex.ItemNotExistsException;

/**
 * Clase que representa a los artculos de usuario
 * 
 * 
 *
 */
public class MyArticle implements Comparable {

	/* Atributos de myArticle */
	private String title;

	private String url;
	
	private SortedSet<String> authors;

	private String id;

	private int priority;

	private SortedSet<String> tags;

	/* Prioridades para MyArticle */
	public static int TOP_PRIORITY = 5;

	public static int MUST_READ = 4;

	public static int I_WILL_READ = 3;

	public static int DONT_WANT_READ = 2;

	public static int IVE_ALREADY_READ_IT = 1;

	public MyArticle(String title, String url, SortedSet<String> authors,
			SortedSet<String> tags, int priority)
			throws InvalidParameterException {
		this(UUID.randomUUID().toString(), title, url, authors, tags, priority);
	}

	public MyArticle(String id, String title, String url, SortedSet<String> authors,
			SortedSet<String> tags, int priority)
			throws InvalidParameterException {

		if (authors == null) {
			throw new InvalidParameterException(
					"Authors param must be not null");
		}
		if (tags == null) {
			throw new InvalidParameterException("Tags param must be not null");
		}
		if (!(priority <= TOP_PRIORITY && priority >= IVE_ALREADY_READ_IT)) {
			throw new InvalidParameterException(
					"Priority param must be correct value. Use static fields "
							+ "from this class. In example, MyArticle.TOP_PRIORITY");
		}

		this.id = id;

		this.title = title;
		this.url = url;
		this.authors = authors;
		this.tags = tags;
		this.priority = priority;
	}

	/**
	 * 
	 * Aade una categora al artculo
	 * @param tagName Categora a aadir
	 * @throws ItemAlreadyExistsException Si la categora ya exista
	 */
	public void addTag(String tagName) throws ItemAlreadyExistsException {
		boolean enc = false;
		for (String t : this.tags) {
			enc = t.equals(tagName);
		}
		if (!enc) {
			this.tags.add(tagName);
		} else {
			throw new ItemAlreadyExistsException("Tag " + tagName
					+ " already exists");
		}
	}

	public boolean equals(MyArticle otherMyArticle) {
		return this.id.equals(otherMyArticle.getId());
	}

	public SortedSet<String> getAuthors() {
		return authors;
	}

	public String getId() {
		return id;
	}

	public int getPriority() {
		return this.priority;
	}

	public SortedSet<String> getTags() {
		return tags;
	}

	public String getTitle() {
		return title;
	}

	/**
	 * Elimina una categora del artculo
	 * 
	 * @param tagName Categora a eliminar
	 * @throws ItemNotExistsException Si la categora no estaba asociada
	 */
	public void removeTag(String tagName) throws ItemNotExistsException {
		boolean enc = false;
		String tagFound = null;
		for (String t : this.tags) {
			enc = t.equals(tagName);
			if (enc) {
				tagFound = t;
			}
		}
		if (enc) {
			this.tags.remove(tagFound);
		} else {
			throw new ItemNotExistsException("Tag " + tagName + " not exists");
		}
	}

	public void setAuthors(SortedSet<String> authors) {
		this.authors = authors;
	}

	public void setPriority(int priority) throws InvalidParameterException {
		if (!(priority <= TOP_PRIORITY && priority >= IVE_ALREADY_READ_IT)) {
			throw new InvalidParameterException(
					"Priority param must be correct value. Use static fields "
							+ "from this class. In example, MyArticle.TOP_PRIORITY");
		}
		this.priority = priority;
	}

	public void setTags(SortedSet<String> tags) {
		this.tags = tags;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * 
	 * Los artculos se comparan por su ttulo
	 * @param o Artculo con el cual comparar
	 */
	public int compareTo(Object o) {
		return this.title.compareTo(((MyArticle) o).getTitle());
	}

	public void setUrl(String url) {
		this.url = url;
	}
	
	public String getUrl() {
		return this.url;
	}

}
